@@ -1,8 +1,9 @@
 /*
+ * Copyright 2012-2014 M. Andersen and L. Vandenberghe.
  * Copyright 2010-2011 L. Vandenberghe.
  * Copyright 2004-2009 J. Dahl and L. Vandenberghe.
  *
- * This file is part of CVXOPT version 1.1.4.
+ * This file is part of CVXOPT version 1.1.7.
  *
  * CVXOPT is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +25,7 @@
 
 PyDoc_STRVAR(fftw__doc__, "Interface to the FFTW3 library.\n");
 
-extern void zscal_(int *n, complex *alpha, complex *x, int *incx);
+extern void zscal_(int *n, double complex *alpha, double complex *x, int *incx);
 extern void dscal_(int *n, double *alpha, double *x, int *incx);
 
 static char doc_dft[] =
