@@ -1,8 +1,9 @@
 /*
+ * Copyright 2012-2014 M. Andersen and L. Vandenberghe.
  * Copyright 2010-2011 L. Vandenberghe.
  * Copyright 2004-2009 J. Dahl and L. Vandenberghe.
  *
- * This file is part of CVXOPT version 1.1.4.
+ * This file is part of CVXOPT version 1.1.7.
  *
  * CVXOPT is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +32,7 @@
 typedef union {
   double d;
   int_t i;
-  complex z;
+  double complex z;
 } number;
 #endif
 
